unit Main;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls;

type
  TForm1 = class(TForm)
    LabeledEdit1: TLabeledEdit;
    LabeledEdit2: TLabeledEdit;
    BitBtn1: TBitBtn;
    Memo1: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
  private
    { Private 錾 }
  public
    { Public 錾 }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses system.Math;

var
  LDBL_EPSILON: extended;

function Landen_Transform(phi, parameter: extended; var F, Fk, E, Ek: extended): integer;
var
  two_n: extended;
  a, g, a_old, g_old: extended;
  tan_2n_phi, sum, integral: extended;
  n : integer;
begin
  two_n := 1.0;
  a := 1.0;
  g := sqrt(1.0 - parameter);
  sum := 2.0 * (2.0 - parameter);
  integral := 0.0;
  n := 0;
  repeat
    tan_2n_phi := tan(two_n * phi);
    sum := sum - two_n * (a - g) * (a - g);
    two_n := two_n + two_n;
    phi := phi - arctan((a - g) * tan_2n_phi / (a + g * tan_2n_phi * tan_2n_phi)) / two_n;
    integral := integral + (a - g) * sin(two_n * phi);
    g_old := g;
    a_old := a;
    a := (g_old + a_old) / 2;
    g := sqrt(g_old * a_old);
    inc(n);
  until (abs(a_old - g_old) <= (a_old * LDBL_EPSILON));
  F := phi / g;
  Fk := PI / 2 / g;
  E :=  1 / 2 * integral + 1 / 4 * sum * phi / g;
  Ek := PI / 8 / a * sum;
  result := n;
end;

procedure Elliptic_Integral_Second_Kind(phi, m: extended; var E, Ek: extended);
var
  F, Fk: extended;
  n, i: integer;
begin
  if m = 1 then begin
    if (abs(phi) <= PI / 2) then begin
      E := sin(phi);
      exit;
    end;
    n := trunc((abs(phi) + PI / 2) / PI);
    E := n + n + sin(abs(phi) - n * PI);
    if phi < 0 then E := - E;
    exit;
  end;
// vZ
  i := Landen_Transform(abs(phi), m, F, Fk, E, Ek);
  Form1.Memo1.Lines.Append('Landen Loop ' + intTostr(i));
  E := sign(phi) * E;
  Form1.Memo1.Lines.Append('@Q@@' + floatTostr(E));
// 90vZ
  n := trunc((phi + PI / 2) / PI );
  phi := phi - n * PI;
  n := n + n;
  i := Landen_Transform(abs(phi), m, F, Fk, E, Ek);
  Form1.Memo1.Lines.Append('Landen Loop ' + intTostr(i));
  E := sign(phi) * E  + n * Ek;
end;

procedure Elliptic_Integral_First_Kind(phi, m: extended; var F, K: extended);
var
  E, Ek: extended;
  n, i: integer;
begin
  if  m = 1.0 then begin
    if abs(phi) >= PI / 2 then begin
      F := infinity;
      if phi < 0 then F := -F;
      exit;
    end;
    Ek := tan(Phi);
    F := sign(Phi) * ln(abs(Ek) + sqrt(1.0 + Ek * Ek));
    exit;
  end;
// vZ
  i := Landen_Transform(abs(phi), m, F, K, E, Ek);
  Form1.Memo1.Lines.Append('Landen Loop ' + intTostr(i));
  F := sign(phi) * F;
  Form1.Memo1.Lines.Append('@P@@' + floatTostr(F));
// 90vZ
  n := trunc((phi + PI / 2) / PI);
  phi := phi - n * PI;
  n := n + n;
  i := Landen_Transform(abs(phi), m, F, K, E, Ek);
  Form1.Memo1.Lines.Append('Landen Loop ' + intTostr(i));
  F := sign(phi) * F + n * K;
end;

procedure TForm1.BitBtn1Click(Sender: TObject);
var
  F, K, E, Ek: extended;
  kin, m : extended;
  deg, phi: extended;
  ch: integer;
begin
  val(labelededit1.Text, kin, ch);
  if ch <> 0 then begin
    application.MessageBox('Sk̒lɊԈႢ܂B','',0);
    exit;
  end;
  if (kin < 0) or (kin > 1) then begin
    application.MessageBox('Sk̒l͈͊OłB','',0);
    exit;
  end;
  val(labelededit2.Text, deg, ch);
  if ch <> 0 then begin
    application.MessageBox('ϕ͈̔̓̒lɊԈႢ܂B','',0);
    exit;
  end;
  memo1.Clear;
  m := kin * kin;                         // k^2
  phi := deg / 180 * pi;
  Elliptic_Integral_First_Kind(phi, m, F, K);
  Memo1.Lines.Append('@P@90@' + floatTostr(F));
  Elliptic_Integral_Second_Kind(phi, m, E, Ek);
  Memo1.Lines.Append('@Q@90@' + floatTostr(E));
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  one, tmp: extended;
begin
  one := 1;
  LDBL_EPSILON := one;
  repeat
    LDBL_EPSILON := LDBL_EPSILON / 2;
    tmp := LDBL_EPSILON + one;
  until tmp = one;
  LDBL_EPSILON := LDBL_EPSILON * 2;
  memo1.Clear;
  memo1.Lines.Append('PAQȉ~ϕ');
end;

end.
